library dpa.table;

import 'dart:mirrors';
import 'dpa_annotations.dart' as annotations;

class Table {
  
  Type type;
  String schema;
  Map<String,Type> _columns=new Map<String,Type>();
  Map<String,String> _columns_names=new Map<String,String>();
  List<String> _indexes=new List<String>();
  List<String> _primary_keys=new List<String>();
  
  Table(this.type);
  
  Table compute(){
   _compute();
    return this;
  }
  
  Map<String,Type> get columns => _columns;
  Map<String,String> get columnsNames => _columns_names;
  List<String> get indexes => _indexes;
  List<String> get primary_keys => _primary_keys;
  
  _compute(){
    ClassMirror cm = reflectClass(this.type);
    
    for(var im in cm.metadata){
      if(im.reflectee.runtimeType == annotations.Schema){
        schema = im.reflectee.toString();
      }
    }
    for (var declaration in cm.declarations.values) {
      if (declaration is VariableMirror) {
        TypeMirror declaredType = declaration.type;
            //print("common.reflect.getFieldsMap: hasReflectedType: "+declaredType.hasReflectedType.toString());
            //print("common.reflect.getFieldsMap: reflectedType: "+declaredType.reflectedType.toString());
            //print("common.reflect.getFieldsMap: variable name: "+MirrorSystem.getName(declaration.simpleName));
        if(declaredType.hasReflectedType == true){
          _columns.putIfAbsent(MirrorSystem.getName(declaration.simpleName), ()=>declaredType.reflectedType);
          _columns_names.putIfAbsent(MirrorSystem.getName(declaration.simpleName), () => MirrorSystem.getName(declaration.simpleName));
          List<InstanceMirror> list = declaration.metadata;
          if(!list.isEmpty){
            list.forEach((im) => __compute(declaration, im));
          }
        }
      }
    }
  }
  __compute(VariableMirror vm, InstanceMirror im){
    if(im.reflectee.runtimeType == annotations.Index) {
      _indexes.add(MirrorSystem.getName(vm.simpleName));
    }
    if(im.reflectee.runtimeType == annotations.Primarykey) {
      _primary_keys.add(MirrorSystem.getName(vm.simpleName));
    }
    if(im.reflectee.runtimeType == annotations.Column) {
      _columns_names.remove(MirrorSystem.getName(vm.simpleName));
      _columns_names.putIfAbsent(MirrorSystem.getName(vm.simpleName), () => im.reflectee.toString());
    }
    
  }
  String toString(){
    String s = "Schema: "+schema+"\n";
    s+="Columns Mapping:\n";
    _columns_names.forEach((_string,_m) => s+='\t'+_string+':'+_m+'\n');
    s+="Columns:\n";
    _columns.forEach((_string,_type) => s+='\t'+_string+':'+_type.toString()+'\n');
    s+="Primary keys:\n";
    _primary_keys.forEach((_i) => s+='\t$_i\n');
    s+="Indexes:\n";
    _indexes.forEach((_i) => s+='\t$_i\n');
    s+="\n";
    return s;
  }
}